package edu.illinois.reex.exploration;

import java.util.TreeSet;

import edu.illinois.reex.scheduling.ChoiceType;
import edu.illinois.reex.scheduling.Scheduler;

public class Data {

    private static final String EMPTY = "";
    private static final String GET_INT = "getInt";
    private static final String DATA_CLASS_NAME = Data.class.getName();

    /**
     * Creates a int data choice point (program will be explored for choices)
     * 
     * @param max
     * @return the chosen int between 0 (inclusive) and max (exclusive).
     */
    public static int getInt(int max) {
        String maxString = EMPTY + max;
        Scheduler.beforeFieldAccess(true, DATA_CLASS_NAME, GET_INT, maxString);
        Scheduler.afterFieldAccess(true, DATA_CLASS_NAME, GET_INT, maxString);
        TreeSet<Integer> intChoices = new TreeSet<Integer>();
        for (int i = 0; i < max; i++) {
            intChoices.add(i);
        }
        return (Integer) Scheduler.choose(intChoices, ChoiceType.DATA);
    }

    /**
     * Creates a boolean data choice point (program will be explored for
     * choices)
     * 
     * @return <code>true</code> or <code>false</code>.
     */
    public static boolean getBoolean() {
        return getInt(2) == 1;
    }

}
