package edu.illinois.reex.exploration;

import org.junit.runner.notification.RunNotifier;
import org.junit.runners.BlockJUnit4ClassRunner;
import org.junit.runners.model.FrameworkMethod;
import org.junit.runners.model.InitializationError;

import edu.illinois.reex.ReExProperties;
import edu.illinois.reex.scheduling.Scheduler;

/**
 * ReEx runner for JUnit4 tests.
 * 
 * @author Vilas Jagannath <vbangal2@illinois.edu>
 * @author Matt Kirn <kirn1@illinois.edu>
 * 
 */
public class JUnit4ReExRunner extends BlockJUnit4ClassRunner {

    private static final String DOT = ".";

    public JUnit4ReExRunner(Class<?> klass) throws InitializationError {
        super(klass);
    }

    @Override
    protected void runChild(FrameworkMethod method, RunNotifier notifier) {
        boolean stopOnFirstError = Boolean.parseBoolean(ReExProperties.getInstance().getProperty(ReExProperties.STOP_ON_FIRST_ERROR_KEY));

        Scheduler.startingExploration(getTestClass().getName() + DOT + method.getName());

        JUnit4WrappedRunNotifier wrappedNotifier = new JUnit4WrappedRunNotifier(notifier);
        wrappedNotifier.testExplorationStarted();

        while (Scheduler.canExecuteMoreSchedules()) {
            Scheduler.startingScheduleExecution();
            super.runChild(method, wrappedNotifier);
            if (wrappedNotifier.isTestFailed()) {
                wrappedNotifier.getFailure().getException().printStackTrace();
                Scheduler.failureDetected(wrappedNotifier.getFailure().getMessage());
                if(stopOnFirstError) {                    
                    break;
                }
            }
            Scheduler.completedScheduleExecution();
        }

        wrappedNotifier.testExplorationFinished();

        Scheduler.completedExploration();
    }

}
