package edu.illinois.reex.exploration;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;

import edu.illinois.reex.scheduling.Scheduler;

public class MainReExRunner {

    private static void printUsageAndExit(String errorMsg) {
        System.out.println("Error: " + errorMsg);
        System.out.println("Usage: MainReExRunner <class-with-main-method> [<args-for-main-method>]");
        System.out.println("Example: MainReExRunner edu.illinois.reex.test.MainTest");
        System.exit(1);
    }

    private static final String DOT = ".";
    private static final String MAIN = "main";

    public static void main(String[] args) {
        if (args.length < 1) {
            printUsageAndExit("expected a main class");
        }
        try {
            Class<?> mainClass = Class.forName(args[0]);
            Method mainMethod = mainClass.getMethod(MAIN, String[].class);
            ArrayList<String> mainMethodArgs = new ArrayList<String>();
            for (int i = 1; i < args.length; i++) {
                mainMethodArgs.add(args[i]);
            }
            Object[] invokeArgs = new Object[1];
            invokeArgs[0] = mainMethodArgs.toArray(new String[mainMethodArgs.size()]);

            Scheduler.startingExploration(args[0] + DOT + MAIN);
            while (Scheduler.canExecuteMoreSchedules()) {
                Scheduler.startingScheduleExecution();
                mainMethod.invoke(null, invokeArgs);
                Scheduler.completedScheduleExecution();
            }
        } catch (InvocationTargetException e) {
            e.getCause().printStackTrace();
            Scheduler.failureDetected(null);
        } catch (ClassNotFoundException e) {
            printUsageAndExit("couldn't find the class specified, please check the classpath");
        } catch (SecurityException e) {
            printUsageAndExit("insufficient permissions");
        } catch (NoSuchMethodException e) {
            printUsageAndExit("main method does not exist in the class specified");
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
            printUsageAndExit("unexpected error");
        } catch (IllegalAccessException e) {
            e.printStackTrace();
            printUsageAndExit("unexpected error");
        } finally {
            Scheduler.completedExploration();
        }
    }

}
