package edu.illinois.reex.scheduling.filtering;

import java.util.SortedSet;

import edu.illinois.reex.ReExProperties;
import edu.illinois.reex.listeners.ExplorationListenerAdapter;
import edu.illinois.reex.listeners.Listeners;
import edu.illinois.reex.scheduling.ChoiceType;
import edu.illinois.reex.scheduling.ThreadInfo;

/**
 * Filters thread choices (based on the location they are executing).
 * 
 * @author Vilas Jagannath <vbangal2@illinois.edu>
 * 
 */
public class LocationBasedPreemptionFilter extends ExplorationListenerAdapter implements SchedulingFilter {

    private static final String MODE_DEFAULT = "METHOD_ALL";

    /* Mode used to filter choices based on the allowed locations */
    private final LocationFilterMode mode;

    /* The thread that was last chosen/executed */
    private ThreadInfo lastChoice;

    public LocationBasedPreemptionFilter() {

        /* Setting the mode property */
        String modeString = ReExProperties.getInstance().getProperty(ReExProperties.MODE_KEY, MODE_DEFAULT);
        this.mode = LocationFilterMode.valueOf(modeString);

        this.lastChoice = null;
        Listeners.addListener(this);
    }

    @Override
    public SortedSet<? extends Object> filterChoices(SortedSet<? extends Object> choices, ChoiceType choiceType) {
        /*
         * If preemption is not possible, return the choices without any
         * filtering
         */
        if (choiceType != ChoiceType.THREAD_TO_SCHEDULE || choices.size() == 1 || !choices.contains(this.lastChoice)) {
            return choices;
        }
        return mode.filter((SortedSet<ThreadInfo>) choices, this.lastChoice);
    }

    @Override
    public void choiceMade(Object choice) {
        if(choice instanceof ThreadInfo) {
            this.lastChoice = (ThreadInfo) choice;
        }
    }

    public LocationFilterMode getMode() {
        return mode;
    }

}
