package edu.illinois.reex.scheduling.strategy;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;

import edu.illinois.reex.scheduling.ChoiceType;

/**
 * Performs a depth first exploration of all possible schedules.
 * 
 * @author Vilas Jagannath <vbangal2@illinois.edu>
 * 
 */
public class DepthFirstStrategy extends SchedulingStrategy {

    protected List<Integer> choicesMade;
    protected int choiceIndex;
    protected int deepestChoiceWithMoreOptions;
    protected boolean firstSchedule;

    public DepthFirstStrategy() {
        startingExploration();
    }

    @Override
    public void startingExploration() {
        this.choicesMade = new ArrayList<Integer>();
        this.firstSchedule = true;
        startingScheduleExecution();
    }

    @Override
    public void startingScheduleExecution() {
        this.choiceIndex = -1;
        this.deepestChoiceWithMoreOptions = -1;
    }

    @Override
    public void completedScheduleExecution() {
        while (this.choicesMade.size() - 1 > this.deepestChoiceWithMoreOptions) {
            this.choicesMade.remove(this.choicesMade.size() - 1);
        }
        this.firstSchedule = false;
    }

    @Override
    public boolean canExecuteMoreSchedules() {
        return this.firstSchedule || !this.choicesMade.isEmpty();
    }

    @Override
    public Object choose(SortedSet<? extends Object> objectChoices, ChoiceType choiceType) {

        /*
         * Making next choice
         */
        this.choiceIndex++;

        /*
         * Assume we will schedule the first one.
         */
        int chosenObjectIndex = 0;

        /*
         * If we have encountered this choice before, make the same choice
         * unless this is the point where we need to make the next choice to
         * explore a new schedule.
         */
        if (this.choiceIndex <= this.choicesMade.size() - 1) {
            chosenObjectIndex = this.choicesMade.get(this.choiceIndex);
            if (this.choiceIndex == this.choicesMade.size() - 1) {
                chosenObjectIndex++;
                this.choicesMade.set(this.choiceIndex, chosenObjectIndex);
            }
        } else {
            this.choicesMade.add(chosenObjectIndex);
        }

        /*
         * If more choices remain, mark this as the deepest choice point.
         */
        if (chosenObjectIndex < objectChoices.size() - 1) {
            this.deepestChoiceWithMoreOptions = this.choiceIndex;
        }

        /*
         * Find the chosen object and return it
         */
        return getChosenObject(chosenObjectIndex, objectChoices);
    }

    @Override
    public List<Integer> getChoicesMadeDuringThisSchedule() {
        return choicesMade.subList(0, choiceIndex + 1);
    }

}
