package edu.illinois.reex.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;

import edu.illinois.reex.exploration.Data;
import edu.illinois.reex.exploration.JUnit4ReExRunner;

@RunWith(JUnit4ReExRunner.class)
public class DataChoiceTests {

    private static int justDataChoiceSchCount = 7;
    private static int threadAndDataChoiceSchCount = 15;

    @Test
    public void testJustDataChoice() {
        try {
            Data.getInt(0);
            fail("Should throw IllegalArgumentException");
        } catch (IllegalArgumentException exp) {
        }
        assertEquals(0, Data.getInt(1));
        int intChoice = Data.getInt(4);
        assertTrue(intChoice < 4 && intChoice >= 0);
        boolean boolChoice = Data.getBoolean();
        if (justDataChoiceSchCount == 0) {
            assertEquals(3, intChoice);
            assertEquals(true, boolChoice);
        } else if (justDataChoiceSchCount == 7) {
            assertEquals(0, intChoice);
            assertEquals(false, boolChoice);
        }
        justDataChoiceSchCount--;
    }

    @Test
    public void testThreadAndDataChoice() throws InterruptedException {
        try {
            Data.getInt(0);
            fail("Should throw IllegalArgumentException");
        } catch (IllegalArgumentException exp) {
        }
        assertEquals(0, Data.getInt(1));

        final List<Integer> intChoiceBox = new ArrayList<Integer>();
        Thread intChoiceThread = new Thread(new Runnable() {

            @Override
            public void run() {
                int intChoice = Data.getInt(4);
                assertTrue(intChoice < 4 && intChoice >= 0);
                intChoiceBox.add(intChoice);
            }
        });

        final List<Boolean> boolChoiceBox = new ArrayList<Boolean>();
        Thread boolChoiceThread = new Thread(new Runnable() {

            @Override
            public void run() {
                boolean boolChoice = Data.getBoolean();
                boolChoiceBox.add(boolChoice);
            }
        });

        intChoiceThread.start();
        boolChoiceThread.start();
        intChoiceThread.join();
        boolChoiceThread.join();

        if (threadAndDataChoiceSchCount == 0) {
            assertTrue(intChoiceBox.size() == boolChoiceBox.size() && boolChoiceBox.size() == 1);
            assertEquals(3, (int) intChoiceBox.get(0));
            assertEquals(true, boolChoiceBox.get(0));
        } else if (threadAndDataChoiceSchCount == 15) {
            assertTrue(intChoiceBox.size() == boolChoiceBox.size() && boolChoiceBox.size() == 1);
            assertEquals(0, (int) intChoiceBox.get(0));
            assertEquals(false, boolChoiceBox.get(0));
        }
        threadAndDataChoiceSchCount--;
    }
}
