package edu.illinois.reex.test;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.junit.Test;
import org.junit.runner.RunWith;

import edu.illinois.reex.exploration.JUnit4ReExRunner;

@RunWith(JUnit4ReExRunner.class)
public class JavaUtilConcurrentTests {

    private int counter;

    @Test
    public void testReentrantLock() throws InterruptedException {
        counter = 0;
        ReentrantLock lock = new ReentrantLock();
        Thread t1 = new Thread(new LockProtectedIncrement(lock));
        Thread t2 = new Thread(new LockProtectedIncrement(lock));
        t1.start();
        t2.start();
        t1.join();
        t2.join();
        assert 2 == counter;
    }

    @Test
    public void testDeadlock() throws InterruptedException {
        ReentrantLock lock1 = new ReentrantLock();
        ReentrantLock lock2 = new ReentrantLock();
        Thread t1 = new Thread(new AcquireTwoLocks(lock1, lock2));
        Thread t2 = new Thread(new AcquireTwoLocks(lock2, lock1));
        t1.start();
        t2.start();
        t1.join();
        t2.join();
    }

    private final class LockProtectedIncrement implements Runnable {

        private final Lock lock;

        public LockProtectedIncrement(Lock lock) {
            this.lock = lock;
        }

        @Override
        public void run() {
            lock.lock();
            try {
                counter++;
            } finally {
                lock.unlock();
            }
        }
    }

    private final class AcquireTwoLocks implements Runnable {

        private final Lock lock1;
        private final Lock lock2;

        public AcquireTwoLocks(Lock lock1, Lock lock2) {
            this.lock1 = lock1;
            this.lock2 = lock2;
        }

        @Override
        public void run() {
            lock1.lock();
            try {
                lock2.lock();
                try {

                } finally {
                    lock2.unlock();
                }
            } finally {
                lock1.unlock();
            }
        }
    }

    public static void main(String[] args) throws InterruptedException {
        new JavaUtilConcurrentTests().testDeadlock();
    }

}
