package jBench.programs.buffer;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.LinkedList;
import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;

import edu.illinois.reex.exploration.JUnit4ReExRunner;

/**
 * 
 * @author MTMT
 * 
 */
@RunWith(JUnit4ReExRunner.class)
public class BufferTest {

  public static void main(String[] args) throws InterruptedException {
  }

  @Test
  public void testProducer2() throws Exception {
    Integer[] producer1Items = { 1, 3, 5 };
    Integer[] producer2Items = { 7, 8, 9 };

    Buffer<Integer> buffer = new Buffer<Integer>(20);

    List<Integer> producer1List = createProducerList(producer1Items);
    List<Integer> producer2List = createProducerList(producer2Items);

    Producer<Integer> producer1 = new Producer<Integer>(buffer, producer1List);
    Producer<Integer> producer2 = new Producer<Integer>(buffer, producer2List);

    Thread pt1 = new Thread(producer1);
    Thread pt2 = new Thread(producer2);

    pt1.start();
    pt2.start();

    pt1.join();
    pt2.join();

    assertEquals(producer1Items.length + producer2Items.length, buffer.size());
  }

  @Test
  public void testProducer2ConsumerEmpty() throws Exception {
    Buffer<Integer> buffer = new Buffer<Integer>(3);

    Producer<Integer> producer1 = new Producer<Integer>(buffer, new LinkedList<Integer>());
    Producer<Integer> producer2 = new Producer<Integer>(buffer, new LinkedList<Integer>());

    Consumer<Integer> consumer = new Consumer<Integer>(buffer, 0);

    Thread pt1 = new Thread(producer1);
    Thread pt2 = new Thread(producer2);
    Thread ct = new Thread(consumer);

    pt1.start();
    pt2.start();
    ct.start();

    pt1.join();
    pt2.join();
    ct.join();

    assertEquals(consumer.getConsumedItems().size(), 0);
  }

  @Test
  public void testProducer2Consumer() throws Exception {
    Integer[] producer1Items = { 1, 3, 5 };
    Integer[] producer2Items = { 7, 8, 9 };

    Buffer<Integer> buffer = new Buffer<Integer>(3);
    List<Integer> original = new LinkedList<Integer>();

    List<Integer> producer1List = createProducerList(producer1Items);
    List<Integer> producer2List = createProducerList(producer2Items);

    original.addAll(producer1List);
    original.addAll(producer2List);

    Producer<Integer> producer1 = new Producer<Integer>(buffer, producer1List);
    Producer<Integer> producer2 = new Producer<Integer>(buffer, producer2List);

    Consumer<Integer> consumer = new Consumer<Integer>(buffer, original.size());

    Thread pt1 = new Thread(producer1);
    Thread pt2 = new Thread(producer2);
    Thread ct = new Thread(consumer);

    pt1.start();
    pt2.start();
    ct.start();

    pt1.join();
    pt2.join();
    ct.join();

    assertEquals(consumer.getConsumedItems().size(), original.size());
    // producer1List.addAll(producer2List);
    // assertTrue(compareAsSets(producer1List, consumer.getConsumedItems()));
  }

  @Test
  public void testProducerConsumer() throws Exception {
    Integer[] producerItems = { 1, 3, 5, 7, 9 };

    Buffer<Integer> buffer = new Buffer<Integer>(3);
    List<Integer> original = createProducerList(producerItems);

    List<Integer> producerList = createProducerList(producerItems);

    Producer<Integer> producer = new Producer<Integer>(buffer, producerList);
    Consumer<Integer> consumer = new Consumer<Integer>(buffer, producerList.size());

    Thread pt = new Thread(producer);
    Thread ct = new Thread(consumer);

    pt.start();
    ct.start();

    pt.join();
    ct.join();

    assertEquals(consumer.getConsumedItems().size(), original.size());
    assertTrue(compareAsSets(original, consumer.getConsumedItems()));
    assertTrue(compareOrder(original, consumer.getConsumedItems()));
  }

  private boolean compareOrder(List<Integer> original,
                               List<Integer> consumedItems) {
    if (original.size() != consumedItems.size())
      return false;

    for (int i = 0; i < original.size(); i++)
      if (original.get(i) != consumedItems.get(i))
        return false;

    return true;
  }

  private static List<Integer> createProducerList(Integer[] items) {
    List<Integer> producerList = new LinkedList<Integer>();
    for (Integer item : items)
      producerList.add(item);
    return producerList;
  }

  private static boolean compareAsSets(List<Integer> listToProduce,
                                       List<Integer> listOfConsumed) {

    if (listToProduce.size() != listOfConsumed.size())
      return false;
    return listToProduce.containsAll(listOfConsumed);
  }

}
