package jBench.programs.norollback;

import java.util.HashMap;

import junit.framework.Assert;

import org.junit.Test;
import org.junit.runner.RunWith;

import edu.illinois.reex.exploration.JUnit4ReExRunner;

@RunWith(JUnit4ReExRunner.class)
public class NoRollbackTest {

    static class TestThread extends Thread {
        public int ticketId;
        public int bankId;

        public TestThread(int ticketId, int bankId) {
            this.ticketId = ticketId;
            this.bankId = bankId;
        }

        @Override
        public void run() {
            Accounts.book(ticketId, bankId);
        }
    };

    static abstract class TestAux {

        protected int numOfThreads;

        public TestAux(int numOfThreads) {
            this.numOfThreads = numOfThreads;
        }

        protected void createTicketsAndAccounts() {
            Accounts.tickets = new HashMap<Integer, Boolean>();
            Accounts.accounts = new HashMap<Integer, Integer>();
        }

        protected void initTicketsAndAccounts() {
            for (int i = 0; i < numOfThreads; i++) {
                Accounts.tickets.put(i, false);
                Accounts.accounts.put(i, Accounts.BAL);
            }
        }

        protected abstract TestThread[] createThreads();

        protected void startThreads(Thread[] threads) {
            for (Thread t : threads)
                t.start();
        }

        protected void joinThreads(Thread[] threads) throws InterruptedException {
            for (Thread t : threads)
                t.join();
        }

        protected void startAndJoinThreads(Thread[] threads) throws InterruptedException {
            startThreads(threads);
            joinThreads(threads);
        }

        protected abstract boolean invariant();

        @Deprecated
        protected boolean balanceInvariant() {
            return (Accounts.overalBalance() + Accounts.numOfSoldTickets() * Accounts.SUM == Accounts.BAL * numOfThreads);
        }

        public final void genericTest() throws InterruptedException {
            createTicketsAndAccounts();
            initTicketsAndAccounts();

            TestThread[] threads = createThreads();
            startAndJoinThreads(threads);

            if (!invariant())
                Assert.fail("!");
        }

    }

    static class TestAuxDiff extends TestAux {

        public TestAuxDiff(int numOfThreads) {
            super(numOfThreads);
        }

        @Override
        protected TestThread[] createThreads() {
            TestThread[] threads = new TestThread[numOfThreads];

            for (int i = 0; i < numOfThreads; i++)
                threads[i] = new TestThread(i, i);
            return threads;
        }

        @Override
        public boolean invariant() {
            boolean soldInvariant = Accounts.numOfSoldTickets() == numOfThreads;
            boolean invariant = balanceInvariant() && soldInvariant;
            return invariant;
        }
    }

    static class TestAuxSame extends TestAux {

        public TestAuxSame(int numOfThreads) {
            super(numOfThreads);
        }

        @Override
        protected TestThread[] createThreads() {
            TestThread[] threads = new TestThread[numOfThreads];
            for (int i = 0; i < numOfThreads; i++)
                threads[i] = new TestThread(i, 0);
            return threads;
        }

        @Override
        protected boolean invariant() {
            boolean soldInvariant = Accounts.numOfSoldTickets() > 0;
            boolean invariant = balanceInvariant() && soldInvariant;
            return invariant;
        }

    }

    @Test
    public void testDiffThreads1() throws Exception {
        new TestAuxDiff(1).genericTest();
    }

    @Test
    public void testDiffThreads3() throws Exception {
        new TestAuxDiff(3).genericTest();
    }

    //@Test
    public void testDiffThreads5() throws Exception {
        new TestAuxDiff(5).genericTest();
    }

    @Test
    public void test1Threads() throws Exception {
        new TestAuxSame(1).genericTest();
    }

    @Test
    public void test3Threads() throws Exception {
        new TestAuxSame(3).genericTest();
    }

    //@Test
    public void test5Threads() throws Exception {
        new TestAuxSame(5).genericTest();
    }

    public static void main(String[] args) throws Exception {
        new NoRollbackTest().test5Threads();
    }

}
